<?php
# 実行した際に出力が行われるように一部サンプルコード修正しています。コピーして実際に実行してみるなどして参考にしてください。

# 添字配列 - P132

$array1 = [0 => 'foo', 1 => 'bar'];
$array2 = ['foo', 'bar'];
print_r($array1);
print_r($array2);



# 連想配列 - P132

$array = ['hoge' => 'fuga', 'foo' => 'bar'];
print_r($array);



# PHPの配列はすべて値渡しされる - P133

$array1 = ['hoge' => 'fuga'];
$array2 = $array1; // 配列がコピーされる
$array2['hoge'] = 'piyo';
// 変更は$array2に対してのみ行われる
// $array1: ['hoge' => 'fuga']
// $array2: ['hoge' => 'piyo']
print_r($array1);
print_r($array2);

function foo(array $array) {
    $array = ['hoge' => 'piyo'];
}
foo($array1); // $array1は['hoge' => 'fuga']のまま
print_r($array1);



# array_unshift()関数の場合 - P133

$array = [4, 5];
array_unshift($array, 3);
// 返り値として3が返り、$arrayは[3, 4, 5]となる

array_unshift($array, 1, 2);
// 複数の要素を追加することも可能
print_r($array);



# 角括弧構文（[]）の場合 - P133

$array = [1, 2];
$array[] = 3;
// $arrayは[1, 2, 3]となる
// 返り値は単なる代入となるので右辺の値
print_r($array);

$array = ['hoge' => 'fuga'];
$array['foo'] = 'bar';
// ['hoge' => 'fuga', 'foo' => 'bar']
print_r($array);



# +演算子の場合 - P134

$array1 = ['hoge' => 'fuga'];
$array2 = ['hoge' => 'piyo', 'foo' => 'bar'];
print_r($array1 + $array2);
// ['hoge' => 'fuga', 'foo' => 'bar']

$array3 = ['hoge', 'fuga'];
$array4 = ['foo', 'bar'];
print_r($array3 + $array4);
// 同じキーを有していたとみなされるため
// $array4の要素は無視され結果は['hoge', 'fuga']となる



# array_merge()関数の場合 - P134

$array1 = ['hoge' => 'fuga'];
$array2 = ['hoge' => 'piyo', 'foo' => 'bar'];
print_r(array_merge($array1, $array2));
// ['hoge' => 'piyo', 'foo' => 'bar']

$array1 = [3 => 'hoge', 5 => 'fuga'];
$array2 = [3 => 'foo', 5 => 'bar'];
print_r(array_merge($array1, $array2));
// キーが採番しなおされて['hoge', 'fuga', 'foo', 'bar']

$array3 = ['hoge', 'fuga'];
$array4 = ['foo', 'bar'];
print_r(array_merge($array3, $array4));
// キーが採番しなおされて['hoge', 'fuga', 'foo', 'bar']
// となり直感的な結果ではある



# array_merge_recursive()関数の場合 - P134

$array1 = ['hoge' => 'fuga'];
$array2 = ['hoge' => 'piyo', 'foo' => 'bar'];
print_r(array_merge_recursive($array1, $array2));
// ['hoge' => ['fuga', 'piyo'], 'foo' => 'bar']

$array1 = ['hoge' => 'fuga'];
$array2 = ['hoge' => 'fuga', 'foo' => 'bar'];
print_r(array_merge_recursive($array1, $array2));
// ['hoge' => ['fuga', 'fuga'], 'foo' => 'bar']
// 内部的にarray_merge(['fuga'], ['fuga'])が行われている



# in_array()関数の場合 - P135

$array = ['hoge' => 'fuga', 'foo' => 'bar'];
var_dump(in_array('fuga', $array, true)); // true
var_dump(in_array('piyo', $array, true)); // false



# array_search()関数の場合 - P135

$array = ['hoge' => 'fuga', 'foo' => 'bar'];
var_dump(array_search('fuga', $array, true)); // 'hoge'
var_dump(array_search('piyo', $array, true)); // false



# 第３引数に気をつけて - P135

$array = ['hoge', 'fuga', ''];
var_dump(in_array(false, $array, false));
// falseと''が等しいとみなされtrueが返る
var_dump(in_array(false, $array, true)); // false
var_dump(array_search(false, $array, false));
// falseと''が等しいとみなされ''のキーである2が返る

var_dump(array_search(false, $array, true)); // false



# json_encode()関数 - P136

$array1 = ['hoge', 'fuga'];
var_dump(json_encode($array1)); // '["hoge","fuga"]'

$array2 = ['hoge' => 'fuga', 'foo' => 'bar'];
var_dump(json_encode($array2)); // '{"hoge":"fuga","foo":"bar"}'



# json_decode()関数 - P136

$array = '{"hoge":"fuga","foo":"bar"}';
var_dump(json_decode($array, true));
// ['hoge' => 'fuga', 'foo' => 'bar']



# 必ず失敗していないかどうかをチェックしよう - P136

if (JSON_ERROR_NONE !== json_last_error()) {
    throw new \InvalidArgumentException(json_last_error_msg());
}



# narsort()関数 - P137

$versions = ['PHP 7.1.2','PHP 7.1.10','PHP 7.1.1'];
asort($versions);
// PHP 7.1.1, PHP 7.1.10, PHP 7.1.2とソートされる
print_r($versions);

$versions = ['PHP 7.1.2','PHP 7.1.10','PHP 7.1.1'];
natsort($versions);
// PHP 7.1.1, PHP 7.1.2, PHP 7.1.10とソートされる
print_r($versions);



# array_unique()関数 - P137

$array = ['hoge', 'fuga', 'hoge', 'piyo'];
$results = array_unique($array);
// [0 => 'hoge', 1 => 'fuga', 3 => 'piyo']
var_dump($results);

$array = ['hoge', 'fuga', 'hoge', 'piyo'];
$results = json_encode(array_unique($array));
// '["hoge","fuga","piyo"]'ではなく
// '{"0":"hoge","1":"fuga","3":"piyo"}'になってしまう
var_dump($results);

$array = ['hoge', 'fuga', 'hoge', 'piyo'];
$results = json_encode(
    array_values(array_unique($array))
); // '["hoge","fuga","piyo"]'
var_dump($results);



# array_column()関数 - P138

$array = [
    ['id' => 111, 'name' => 'Yamada'],
    ['id' => 222, 'name' => 'Suzuki'],
    ['id' => 333, 'name' => 'Sato'],
];
$results = array_column($array, 'name');
// ['Yamada', 'Suzuki', 'Sato']
print_r($results);

$results = array_column($array, 'name', 'id');
// [111 => 'Yamada', 222 => 'Suzuki', 333 => 'Sato']
print_r($results);

$results = array_column($array, null, 'id');
// [
//     111 => ['id' => 111, 'name' => 'Yamada'],
//     222 => ['id' => 222, 'name' => 'Suzuki'],
//     333 => ['id' => 333, 'name' => 'Sato']
// ]
print_r($results);



# array_filter()関数 - P138

$array = [1, 2, 'hoge', 3, 'fuga'];
$results = array_filter($array, 'is_string');
// [2 => 'hoge', 4 => 'fuga'];
print_r($results);



# array_map()関数 - P138

$array = [1, 2, 3];
$results = array_map(
    function (int $a) {
        return $a * $a;
    },
    $array
);
// [1, 4, 9]
print_r($results);



# array_reduce()関数 - P139

$array = range(1, 10);
$results = array_reduce(
    $array,
    function (int $sum, int $i) {
        $sum += $i;
        return $sum;
    },
    0
);
// 55
var_dump($results);

